#include "LEMuSRMUONIUMScatt.hh"
#include "G4StepStatus.hh"
#include "G4Navigator.hh"
#include "G4TransportationManager.hh"
#include "Randomize.hh"
#include "G4ProductionCutsTable.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
    
using namespace std;

LEMuSRMUONIUMScatt::  LEMuSRMUONIUMScatt(const G4String& name ,
			       G4ProcessType   aType )
 : G4VDiscreteProcess(name, aType)
{}


LEMuSRMUONIUMScatt:: ~LEMuSRMUONIUMScatt()
{}


G4VParticleChange* LEMuSRMUONIUMScatt::PostStepDoIt(
                                               const G4Track& trackData,
                                               const G4Step& aStep )
{
  fParticleChange.Initialize(trackData);
  
  // tao :: Get Time
  G4double itime = trackData.GetProperTime();
  G4double gtime = trackData.GetGlobalTime();
  G4double ftime = trackData.GetDynamicParticle()->GetPreAssignedDecayProperTime(); 
  
  G4double deltatime = ftime - itime;
  fParticleChange.ProposeGlobalTime(deltatime + itime -gtime);
  
  // set position momentum energy
  fParticleChange.ProposePosition(trackData.GetPosition());
  fParticleChange.ProposeMomentumDirection(trackData.GetMomentumDirection());
  fParticleChange.ProposeEnergy(trackData.GetKineticEnergy());
  fParticleChange.ProposeGlobalTime(gtime);
  fParticleChange.ProposeProperTime(itime);
  fParticleChange.ProposeTrackStatus(trackData.GetTrackStatus()) ;
  
  if( CheckCondition(aStep))
    {
      //      fParticleChange.ProposeEnergy(0);
   fParticleChange.ProposePosition(trackData.GetStep()->GetPreStepPoint()->GetPosition());
    
      fParticleChange.ProposePolarization(trackData.GetPolarization());
      //      G4cout<<" at rest "<< trackData.GetStep()->GetPreStepPoint()->GetPosition() <<G4endl;
  fParticleChange.ProposeTrackStatus(fStopButAlive) ;
      
    }  
  else
    {
      fParticleChange.ProposePolarization(trackData.GetPolarization());
    }
  //   fParticleChange.DumpInfo();
  return &fParticleChange;



}





G4bool LEMuSRMUONIUMScatt::CheckCondition( const G4Step& aStep)
{
  G4bool condition=false;
  p_name = aStep.GetTrack()->GetDefinition()->GetParticleName(); // particle name  
  if(p_name == "Mu"&&aStep.GetTrack()->GetVolume()->GetLogicalVolume()->GetName()!="lv_CFOIL"&&aStep.GetTrack()->GetVolume()->GetLogicalVolume()->GetMaterial()->GetName()!="vacuum")
    {
      condition=true;
    }
  
  return condition;
 
}  
  

G4double LEMuSRMUONIUMScatt:: GetMeanFreePath(const G4Track& ,
			   G4double  ,
			   G4ForceCondition* condition
                                                               )
{

  *condition = Forced;
  return DBL_MAX;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void LEMuSRMUONIUMScatt::GetDatas( const G4Step* aStep)
{
      
      // Particle generation according to yields
      particleTable=G4ParticleTable::GetParticleTable();
      particle = particleTable->FindParticle("Mu");
     
      // Set the new dynamic particle
      DP = new G4DynamicParticle(particle,
				 aStep->GetTrack()->GetDynamicParticle()
				 ->GetMomentumDirection(),0.0/*the kinectic energy set to zero*/
				 );
      
      
      DP->SetProperTime(aStep->GetTrack()->GetDynamicParticle()->GetPreAssignedDecayProperTime());
      DP->SetPolarization(aStep->GetTrack()->GetDynamicParticle()->GetPolarization().x(),aStep->GetTrack()->GetDynamicParticle()->GetPolarization().y(),aStep->GetTrack()->GetDynamicParticle()->GetPolarization().z());
      DP->SetPreAssignedDecayProperTime(aStep->GetTrack()->GetDynamicParticle()->GetPreAssignedDecayProperTime());
      

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......



void LEMuSRMUONIUMScatt::PrepareSecondary(const G4Track& track)
{
  aSecondary = new G4Track(DP,track.GetDynamicParticle()->GetPreAssignedDecayProperTime(),track.GetPosition());


;}
